@@ -6,7 +6,7 @@ import torch
 from scipy.special import softmax
 
 from medcat.utils.ml_utils import train_network, eval_network
-from medcat.utils.data_utils import prepare_from_json, encode_category_values, tkns_to_ids, set_all_seeds
+from medcat.utils.data_utils import prepare_from_json, encode_category_values, set_all_seeds
 from medcat.preprocessing.tokenizers import TokenizerWrapperBPE
 from medcat.preprocessing.tokenizers import TokenizerWrapperBERT
 
@@ -34,10 +34,6 @@ class MetaCAT(object):
 
         self.model = None
 
-        # TODO: A shitty solution, make right at some point
-        if not self.save_dir.endswith("/"):
-            self.save_dir = self.save_dir + "/"
-
 
     def train(self, json_path, category_name=None, model_name='lstm', lr=0.01, test_size=0.1,
               batch_size=100, nepochs=20, class_weights=None, cv=0,
@@ -76,9 +72,6 @@ class MetaCAT(object):
             # We already have everything, just get the data
             data, _ = encode_category_values(data, vals=self.category_values)
 
-        # Convert data tkns to ids
-        #data = tkns_to_ids(data, self.tokenizer)
-
         if not fine_tune:
             if model_name == 'lstm':
                 from medcat.utils.models import LSTM
@@ -144,7 +137,7 @@ class MetaCAT(object):
         data = json.load(open(json_path, 'r'))
 
         # Prepare the data
-        data = prepare_from_json(data, self.cntx_left, self.cntx_right, self.tokenizer, lowercase=lowercase, cui_filter=cui_filter,
+        data = prepare_from_json(data, self.cntx_left, self.cntx_right, self.tokenizer, cui_filter=cui_filter,
                 replace_center=replace_center)
 
         # Check is the name there
@@ -156,9 +149,6 @@ class MetaCAT(object):
         # We already have everything, just get the data
         data, _ = encode_category_values(data, vals=self.category_values)
 
-        # Convert data tkns to ids
-        data = tkns_to_ids(data, self.tokenizer)
-
         # Run evaluation
         result = eval_network(self.model, data, max_seq_len=(self.cntx_left+self.cntx_right+1), pad_id=self.pad_id,
                 batch_size=batch_size, device=self.device, ignore_cpos=ignore_cpos, score_average=score_average)
@@ -202,7 +192,7 @@ class MetaCAT(object):
                 # The tokenizer wrapper saving  
                 self.tokenizer.save(self.save_dir, name=tokenizer_name)
             # Save embeddings
-            np.save(open(self.save_dir + "embeddings.npy", 'wb'), np.array(self.embeddings))
+            np.save(os.path.join(self.save_dir, 'embeddings.npy'), np.array(self.embeddings))
 
         # The lstm model is saved during training, don't do it here
         #save the config.
@@ -211,7 +201,7 @@ class MetaCAT(object):
 
     def save_config(self):
         # TODO: Add other parameters, e.g replace_center, ignore_cpos etc.
-        path = self.save_dir + "vars.dat"
+        path = os.path.join(self.save_dir, 'vars.dat')
         to_save = {'category_name': self.category_name,
                    'category_values': self.category_values,
                    'i_category_values': self.i_category_values,
@@ -226,7 +216,7 @@ class MetaCAT(object):
     def load_config(self):
         """ Loads variables of this object
         """
-        path = self.save_dir + "vars.dat"
+        path = os.path.join(self.save_dir, 'vars.dat')
         with open(path, 'rb') as f:
             to_load = pickle.load(f)
 
@@ -252,21 +242,21 @@ class MetaCAT(object):
 
             self.model = LSTM(self.embeddings, self.pad_id, nclasses=nclasses, bid=bid, num_layers=num_layers,
                          input_size=input_size, hidden_size=hidden_size, dropout=dropout)
-            path = self.save_dir + "lstm.dat"
+            path = os.path.join(self.save_dir, 'lstm.dat')
 
         self.model.load_state_dict(torch.load(path, map_location=self.device))
 
     @classmethod
-    def load(cls, save_dir, model='lstm'):
+    def load(cls, save_dir, model='lstm', **kwargs):
         ''' Load from full save
         '''
         mc = cls(save_dir=save_dir)
-        mc._load(model=model)
+        mc._load(model=model, **kwargs)
 
         return mc
 
 
-    def _load(self, model='lstm'):
+    def _load(self, model='lstm', **kwargs):
         """ Loads model and config for this meta annotation
         """
         # Load configuration
@@ -276,14 +266,14 @@ class MetaCAT(object):
         # Load tokenizer if it is None
         if self.tokenizer is None:
             if 'bbpe' in tokenizer_name:
-                self.tokenizer = TokenizerWrapperBPE.load(self.save_dir, name=tokenizer_name)
+                self.tokenizer = TokenizerWrapperBPE.load(self.save_dir, name=tokenizer_name, **kwargs)
             elif 'bert' in tokenizer_name:
-                self.tokenizer = TokenizerWrapperBERT.load(self.save_dir, name=tokenizer_name)
+                self.tokenizer = TokenizerWrapperBERT.load(self.save_dir, name=tokenizer_name, **kwargs)
             else:
                 raise Exception("Tokenizer not supported")
         # Load embeddings if None
         if self.embeddings is None:
-            embeddings = np.load(open(self.save_dir  + "embeddings.npy", 'rb'), allow_pickle=False)
+            embeddings = np.load(os.path.join(self.save_dir, 'embeddings.npy'), allow_pickle=False)
             self.embeddings = torch.tensor(embeddings, dtype=torch.float32)
 
         # Load MODEL
